/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * This file is part of the PSIQS package version 01 from 2016-01-07, 
 * developed by Tilman Neumann, Edemissen, Germany.
 * 
 * Use and redistribution of PSIQS are free for non-commercial purposes, as long as this
 * header is not modified or removed. Any commercial usage of the whole or parts
 * of PSIQS requires the written consent of the author.
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

package de.tilman_neumann.math.base.bigint;

import java.math.BigInteger;

/**
 * i.th root of integers.
 * 
 * @author Tilman Neumann
 */
public class Roots {

	/**
	 * Computes integer values around (i.th root(n)).
	 * Bitwise correction approach.
	 * 
	 * @param n argument
	 * @param i root
	 * @return [lower, upper] int values of i.th root(n)
	 */
	// TODO: Is a Heron-like implementation possible for i.th roots, too?
	public static BigInteger[] ithRoot(BigInteger n, int i) {
		int nBits = n.bitLength();
		int aBits = nBits/i + 1;
		BigInteger ret = BigIntConstants.ZERO;
		for (int bitIdx=aBits-1; bitIdx>=0; bitIdx--) {
			BigInteger next = ret.setBit(bitIdx);
			int cmp = n.compareTo(next.pow(i));
			if (cmp >= 0) {
				ret = next;
				if (cmp == 0) {
					return new BigInteger[] {ret, ret};
				}
			}
		}
		return new BigInteger[] {ret, ret.add(BigInteger.ONE)};
	}
}
