/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * This file is part of the PSIQS package version 01 from 2016-01-07, 
 * developed by Tilman Neumann, Edemissen, Germany.
 * 
 * Use and redistribution of PSIQS are free for non-commercial purposes, as long as this
 * header is not modified or removed. Any commercial usage of the whole or parts
 * of PSIQS requires the written consent of the author.
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

package de.tilman_neumann.math.factor.cfrac.tdiv;

import java.math.BigInteger;

import de.tilman_neumann.math.factor.FactorException;
import de.tilman_neumann.math.factor._congruence.AQPair;

/**
 * Interface for auxiliary factor algorithms to find smooth decompositions of Q's.
 * @author Tilman Neumann
 */
public interface TDiv_CF {
	
	/**
	 * @return the name of this algorithm
	 */
	public String getName();

	/**
	 * Initialize for a new N.
	 * @param N
	 * @param maxSufficientSmoothRest
	 */
	public void initialize(BigInteger N, double maxSufficientSmoothRest);

	/**
	 * Initialize this factorizer for a new k; in particular set the prime base to be used for trial division.
	 * @param kN
	 * @param primeBaseSize the true prime base size (the arrays are preallocated with a bigger length)
	 * @param primesArray prime base in ints
	 * @param primesArray_big prime base in BigIntegers
	 * @throws FactorException 
	 */
	public void initialize(BigInteger kN, int primeBaseSize, int[] primesArray, BigInteger[] primesArray_big) throws FactorException;
	
	/**
	 * Check if Q is smooth (factors completely over the prime base) or "sufficiently smooth" (factors almost over the prime base).
	 *
	 * @param A
	 * @param Q
	 * @return an AQ-pair if Q is at least "sufficiently smooth", null else
	 */
	public AQPair test(BigInteger A, BigInteger Q);
}
